﻿
/**
 * @fileOverview 独自に作成するコンテキストメニューを扱います。
 */

/**
 * コンテキストメニューの枠組みを提供します。
 * @class
 */
function ContextMenu() {
    this.initialise.apply(this, arguments);
}
ContextMenu.prototype = {
	/** コンテキストメニューで使用する PopupItem オブジェクト
	 * @type PopupItem
	 */
	popupItem: null,
	/** コンテキストメニューとして表示するメニュー項目
	 * @type Array
	 * @example
	 * [
	 *     ["MenuItem 1", "IDM_MENUITEM1"], // キャプション, ID
	 *     [["MenuItem 2 with icon", "foobar.png"], "IDM_MENUITEM2"], // アイコン付
	 *     ["-"], // 区切り線
	 *     ["SubMenu 1", [ // サブメニュー
	 *         ["SubMenuItem 1", "IDM_SUBMENUITEM1"],
	 *         ["SubMenuItem 2", "IDM_SUBMENUITEM2"]
	 *     ]]
	 * ]
	 */
	items: null,
	/** メニュー項目がクリックされたときに呼ばれる関数
	 * @type Function
	 */
	onClick: null,
	/** コンテキストメニューが削除されるときに呼ばれる関数
	 * @type Function
	 */
	onRemove: null,
	/** サブメニューが親のコンテキストメニューを指し示す ContextMenu オブジェクト
	 * @type ContextMenu
	 */
	parentMenu: null,
	/** 親のコンテキストメニューがサブメニューを指し示す ContextMenu オブジェクト
	 * @type ContextMenu
	 */
	childMenu:  null,
	/** mousedown イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseDown: function(e) {
		if (this.popupItem) {
			if ((e.target.rel ? e.target.rel : e.target.parentNode.rel) == "ContextMenu") {
				if (e.button == 1) {
					e.preventDefault();
					e.stopPropagation();
				}
			} else {
				this.remove();
			}
		}
	},
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(items) {
		this.items = items;
		window.addEventListener("mousedown", this.onMouseDown.bind(this), false);
	},
	/** コンテキストメニューを表示します。
	 * @param {Number} x1   メニューが表示される原点の X 座標
	 * @param {Number} y1   メニューが表示される原点の Y 座標
	 * @param {Number} [y2] メニューが表示しきれないときに使用される原点の X 座標
	 * @param {Number} [y2] メニューが表示しきれないときに使用される原点の X 座標
	 * @param {ContextMenu} [parentMenu] サブメニューを表示するときに使用される親メニューの ContextMenu オブジェクト
	 */
    show: function(x1, y1, x2, y2, parentMenu) {
		this.popupItem = new PopupItem(null);
		this.popupItem.container.className = "contextMenu";
		var ul = document.createElement("ul");
		ul.contextMenu = this;
		ul.popupItemContainer = this.popupItem.container;
		//ul.id = "" + (new Date).getTime();
		ul.rel = "ContextMenu";
		if (parentMenu) {
			this.parentMenu = parentMenu;
			this.parentMenu.childMenu = this;
		}
		for (var i = 0; i < this.items.length; i++) {
			var a = null;
			if (this.items[i][0] == "-") {
				ul.appendChild(document.createElement("hr"));
			} else {				
				var li = document.createElement("li");
				var span = document.createElement("span");
				span.className = "icon";
				if (this.items[i][0] instanceof Array) {
					span.innerHTML = this.items[i][0][0];
					span.style.backgroundImage = "url(" + this.items[i][0][1] + ")";
				} else {
					span.innerHTML = this.items[i][0];
				}
				span.rel = "ContextMenu";
				li.appendChild(span);
				li.id = this.items[i][1];
				li.rel = "ContextMenu";
				if (this.items[i][1] instanceof Array) {
					li.className = "more";
					li.menuItems = this.items[i][1];
					li.onmouseover = function(e) {
						e.currentTarget.parentNode.contextMenu.removeChild();
						e.currentTarget.className = "selectedMore";
						var menu = new ContextMenu(e.currentTarget.menuItems);
						menu.onClick = e.currentTarget.parentNode.contextMenu.onClick;
						menu.show(
							e.currentTarget.parentNode.popupItemContainer.offsetLeft + e.currentTarget.parentNode.popupItemContainer.offsetWidth - 6,
							e.currentTarget.parentNode.popupItemContainer.offsetTop + e.currentTarget.offsetTop,
							e.currentTarget.parentNode.popupItemContainer.offsetLeft + 6,
							e.currentTarget.parentNode.popupItemContainer.offsetTop + e.currentTarget.offsetTop + e.currentTarget.offsetHeight,
							e.currentTarget.parentNode.contextMenu
						);
					};
				} else {
					li.onmouseover = function(e) {
						if (e.currentTarget.parentNode.contextMenu) e.currentTarget.parentNode.contextMenu.removeChild();
					};
					li.onclick = function(e) { 
						var nodePopup = e.currentTarget.parentNode.popupItemContainer;
						//nodePopup.parentNode.removeChild(nodePopup);
						e.currentTarget.parentNode.contextMenu.remove();
						var ret = e.currentTarget.parentNode.contextMenu.onClick(e.currentTarget.textContent, e.currentTarget.id, e);
					};
					if (li.id.match(/^a:/i)) {
						var a = document.createElement("a");
						a.rel = "ContextMenu";
						a.href = RegExp.rightContext;
						a.appendChild(li);
					}
				}
				if (this.items[i].checked) li.className += " checked";
				
				if (a) {
					ul.appendChild(a);
				} else {
					ul.appendChild(li);
				}
			}
		}
		this.popupItem.content.appendChild(ul);
		if (!x2) x2 = x1;
		if (!y2) y2 = y1;
		this.popupItem.container.style.visibility = "hidden";
		document.body.appendChild(this.popupItem.container);
		if ((window.innerWidth < (x1 + this.popupItem.container.offsetWidth)) && (x2 > this.popupItem.container.offsetWidth)) {
			this.popupItem.container.style.left  = x2 - this.popupItem.container.offsetWidth ;
		} else {
			this.popupItem.container.style.left  = x1;
		}
		if ((window.innerHeight < (y1 + this.popupItem.container.offsetHeight)) && (y2 > this.popupItem.container.offsetHeight)) {
			this.popupItem.container.style.top = y2 - this.popupItem.container.offsetHeight + 12;
		} else {
			this.popupItem.container.style.top = y1;
		}
		this.popupItem.container.style.visibility = "visible";
		this.popupItem.content.focus();
    },
	/** コンテキストメニューを削除します。*/
    remove: function() {
    	var next = this;
    	while (next.parentMenu) {
    		if (next.parentMenu.popupItem.container.parentNode) next.parentMenu.popupItem.container.parentNode.removeChild(next.parentMenu.popupItem.container);
    		var prev = next;
    		var next = next.parentMenu;
    		prev.parentMenu = null;
    	}
		next = this;
    	while (next.childMenu) {
    		if (next.childMenu.popupItem.container.parentNode) next.childMenu.popupItem.container.parentNode.removeChild(next.childMenu.popupItem.container);
    		var prev = next;
    		var next = next.childMenu;
    		prev.childMenu = null;
    	}
    	if (this.popupItem.container.parentNode) this.popupItem.container.parentNode.removeChild(this.popupItem.container);
    	if (this.onRemove) this.onRemove(this);
    },
	/** コンテキストメニューのサブメニューを削除します。*/
    removeChild: function() {
    	var next = this.childMenu;
    	while (next) {
    		next.popupItem.container.parentNode.removeChild(next.popupItem.container);
    		next = next.childMenu;
    	}
    	var li = this.popupItem.content.childNodes[0].childNodes;
    	for (var i = 0; i < li.length; i++) if (li[i].className == "selectedMore") li[i].className = "more";
    	this.childMenu = null;
    },
};

/**
 * スレッドタイトル部のコンテキストメニューを管理します。
 * @static
 */
var ThreadNameContextMenu = {
	/** コンテキストメニューのメニュー項目
	 * @type Array
	 */
	items: ([
		[["コピー",SKIN_PATH + "img/copy.png"],[
			["タイトルと URL", "copyTitleAndUrl"],
			["-"],
			["タイトル", "copyTitle"],
			["URL",     "copyUrl"]
		]],
		["-"],
		[["履歴",SKIN_PATH + "img/history.png"],[]],
		["-"]
		]),
	/** ContextMenu オブジェクト
	 * @type ContextMenu
	 */
	contextMenu: null,
	/** メニュー項目がクリックされたときの処理をします。
	 * @param {String} caption メニュー項目のキャプション
	 * @param {String} id      メニュー項目の ID
	 * @param {event}  e       オリジナルの click イベントの Event オブジェクト
	 */
	onClick: function(caption, id, e) {
		switch (id) {
			case "copyTitleAndUrl":
				ThreadDocument.setClipboard(ThreadDocument.title + "\n" + EXACT_URL + "\n");
				break;
			case "copyTitle":
				ThreadDocument.setClipboard(ThreadDocument.title);
				break;
			case "copyUrl":
				ThreadDocument.setClipboard(EXACT_URL);
				break;
			default:
				/*
				if (e.button == 0) {
					if (id.match(/^thread:/)) location.href = SERVER_URL + RegExp.rightContext + ThreadDocument.option;
					if (id.match(/^board:/))  location.href = RegExp.rightContext;
				} else if (e.button == 1) {
					if (id.match(/^thread:/)) return (SERVER_URL + RegExp.rightContext + ThreadDocument.option);
					if (id.match(/^board:/))  return (RegExp.rightContext);
				}
				*/
		}
	},
	/** contextmenu イベントを処理します。
	 * @param {event} e イベント
	 */
	onContextMenu: function(e) {
		if (e.target.id == "threadName") {
			this.items = this.items.slice(0,4);
			this.items[2][1] = [];
			var boards = [];
			for (var i = 0; i < HistoryManager.items.length; i++) {
				var boardName = HistoryManager.getBoardName(i);
				if (!boards[boardName]) {
					boards[boardName] = [];
					boards[boardName].url = HistoryManager.getBoardUrl(i);
				}
				boards[boardName].unshift([[HistoryManager.getThreadTitle(i), SKIN_PATH + "img/threaditem.png"], "a:" + SERVER_URL + HistoryManager.getUrl(i) + ThreadDocument.option]);
				this.items[2][1].unshift([[HistoryManager.getThreadTitle(i) + " " + "<b>(" + boardName + ")</b>", SKIN_PATH + "img/threaditem.png"], "a:" + SERVER_URL + HistoryManager.getUrl(i) + ThreadDocument.option]);
			}
			for (var i in boards) {
				boards[i].unshift(["-"]);
				boards[i].unshift(["<b>スレッド一覧を開く</b>", "a:" + boards[i].url]);
				this.items.push([[i, SKIN_PATH + "img/folderclosed.png"], boards[i]]);
			}
			this.contextMenu = new ContextMenu(this.items);
			this.contextMenu.onClick = this.onClick;
			this.contextMenu.show(e.target.offsetLeft, e.target.offsetTop + e.target.offsetHeight);
			return true;
		}
	}
};

/**
 * スレッドタイトル部のコンテキストメニューを管理します。
 * @static
 */
var ResNumberContextMenu = {
	/** コンテキストメニューのメニュー項目
	 * @type Array
	 */
	items: ([
		["<b>返信...</b>", "reply"],
		["-"],
		[["コピー",SKIN_PATH + "img/copy.png"],[
			["すべて", "copyAll"],
			["すべて(Jane 形式)", "copyAllJane"],
			["-"],
			["本文",   "copyBody"],
			["名前",   "copyName"],
			["メール", "copyMail"],
			["日付",   "copyDate"],
			["ID",     "copyID"],
			["BeID",   "copyBeID"],
			["-"],
			["このレスのURL",    "copyUrl"]
		]]
		]),
	/** ContextMenu オブジェクト
	 * @type ContextMenu
	 */
	contextMenu: null,
	/** トリガー元の要素
	 * @type element
	 */
	target: null,
	/** メニュー項目がクリックされたときの処理をします。
	 * @param {String} caption メニュー項目のキャプション
	 * @param {String} id      メニュー項目の ID
	 */
	onClick: function(caption, id) {
		switch (id) {
			case "reply":
				if (ResNumberContextMenu.target.href) location.href = ResNumberContextMenu.target.href;
				break;
			case "copyAll":
			case "copyAllJane":
				var container = ResNodes.getParentContainer(this.target, true);
				var number = ResNodes.getNumbers(container, true).first;
				var format = (id == "copyAllJane") ? "Jane" : "2ch";
				//ThreadDocument.setClipboard(ResNodes.getEntireTextByIndex(ResNodes.getIndexByContainer(container), format));
				ThreadDocument.setClipboard(ResNodes.getEntireTextByIndex(parseInt(number.textContent), format));
				break;
			case "copyBody":
			case "copyName":
			case "copyMail":
			case "copyID":
			case "copyBeID":
				var f = {
					"copyBody": ResNodes.getBodyText.bind(ResNodes, true),
					"copyName": ResNodes.getNameText.bind(ResNodes, true),
					"copyMail": ResNodes.getMailText.bind(ResNodes, true),
					"copyID":   ResNodes.getIDText.bind(ResNodes, true),
					"copyBeID": ResNodes.getBeIDText.bind(ResNodes, true) };
				//var container = ResNodes.getParentContainer(this.target);
				var container = ResNodes.getParentContainer(this.target, true);
				container = container.parentNode;
				ThreadDocument.setClipboard(f[id](container));
				break;
			case "copyUrl":
				//var container = ResNodes.getParentContainer(this.target);
				var container = ResNodes.getParentContainer(this.target, true);
				var number = ResNodes.getNumbers(container, true).first;
				ThreadDocument.setClipboard(EXACT_URL + parseInt(number.textContent) + "\n");
				//ThreadDocument.setClipboard(EXACT_URL + ResNodes.getIndexByContainer(container) + "\n");
				break;
		}
	},
	/** 要素の位置とサイズを取得します。
	 * @param  {element} src 要素
	 * @return {object} left, top, width, height, right, bottom の各メンバから成るオブジェクト
	 */
	getRect: function(src) {
		var parent = src;
		var x = 0;
		var y = 0;
		while (parent) {
			x += parent.offsetLeft;
			y += parent.offsetTop;
			parent = parent.offsetParent;
		}
		return {left: x, top: y, width: src.offsetWidth, height: src.offsetHeight,
		        right: x + src.offsetWidth, bottom: y + src.offsetHeight};
	},
	/** contextmenu イベントを処理します。
	 * @param {event} e イベント
	 */
	onContextMenu: function(e) {
		var node = e.target.parentNode;
		if (node) if (node.className == "resNumber") {
			this.target = e.target;
			this.contextMenu = new ContextMenu(this.items);
			this.contextMenu.onClick = this.onClick.bind(this);
			var rc = this.getRect(this.target);
			this.contextMenu.show(rc.left - window.scrollX, rc.top + rc.height - window.scrollY);
			return true;
		}
	}
}

var ChevronContextMenu = {
	/** コンテキストメニューのメニュー項目
	 * @type Array
	 */
	items: ([
		["自動更新", "enableAutoReload"],
		["-"],
		["要約", "digest"],
		["-"],
		["Moreyon(仮)", "enableMoreyon"],
		["スレッド情報...", "showInfo"],
		[["コピー",SKIN_PATH + "img/copy.png"],[
			["タイトルと URL", "copyTitleAndUrl"],
			["-"],
			["タイトル", "copyTitle"],
			["URL",     "copyUrl"]
		]],
		["-"],
		["オプション...", "showOption"]
		//["-"],
		/*[["履歴",SKIN_PATH + "img/history.png"],[]],*/
		]),
	/** ContextMenu オブジェクト
	 * @type ContextMenu
	 */
	contextMenu: null,
	/** メニュー項目がクリックされたときの処理をします。
	 * @param {String} caption メニュー項目のキャプション
	 * @param {String} id      メニュー項目の ID
	 */
	onClick: function(caption, id) {
		switch (id) {
			case "enableAutoReload":
				this.items[0].checked = this.items[0].checked ? false : true;
				AutoReload.enabled = this.items[0].checked;
				break;
			case "digest":
				this.items[2].checked = this.items[2].checked ? false : true;
				Digest.enabled = this.items[2].checked;
				break;
			case "enableMoreyon":
				this.items[4].checked = this.items[4].checked ? false : true;
				Moreyon.enabled = this.items[4].checked;
				break;
			case "showInfo":
				Analyse.analyse();
				break;
			case "copyTitleAndUrl":
				ThreadDocument.setClipboard(ThreadDocument.title + "\n" + EXACT_URL + "\n");
				break;
			case "copyTitle":
				ThreadDocument.setClipboard(ThreadDocument.title);
				break;
			case "copyUrl":
				ThreadDocument.setClipboard(EXACT_URL);
				break;
			case "showOption":
				dialogOptions.show();
				break;
		}
		this.contextMenu = null;
	},
	/** コンテキストメニューが削除されたときの処理をします。*/
	onRemove: function() {
		this.contextMenu = null;
	},
	/** mousedown イベントを処理します。
	 * @param {event} e イベント
	 */
	onMouseDown: function(e) {
		var node = e.target;
		if (node) if (node.id == "chevron") {
			if (this.contextMenu) {
				this.contextMenu.remove();
				this.contextMenu = null;
				return;
			}
			this.items[0].checked = AutoReload.enabled;
			this.items[2].checked = Digest.enabled;
			this.items[4].checked = Moreyon.enabled;
			this.contextMenu = new ContextMenu(this.items);
			this.contextMenu.onClick = this.onClick.bind(this);
			this.contextMenu.onRemove = this.onRemove.bind(this);
			e.cancelBubble = true;
			e.preventDefault();
			this.contextMenu.show(document.body.clientWidth, 32);
		}
	}
}

document.addEventListener("contextmenu", function(e) {
	if (ThreadNameContextMenu.onContextMenu(e) ||
		ResNumberContextMenu.onContextMenu(e)) {
		e.cancelBubble = true;
		e.preventDefault();
	}
}, true);
window.addEventListener("mousedown", ChevronContextMenu.onMouseDown.bind(ChevronContextMenu), true);
