﻿// moreyon.js
// Oreyon モドキ

/**
 * @fileOverview Oreyon ライクにスレッドを可視化する Moreyon（仮称）
 */

/**
 * Moreyon を管理します。
 * @static
 * @property {Boolean} enabled 有効/無効（表示/非表示）の切り替え
 */
var Moreyon = {
	PI2: Math.PI * 2,
	/** SVGCanvas オブジェクト
	 * @type SVGCanvas
	 */
	SVGCanvas: null,
	/** SVGLayer を格納する配列
	 * @type Array
	 */
	SVGLayers: [],
	/** 円の半径
	 * @type Number
	 */
	r: 12,
	/** 一行に表示するアイテム数
	 * @type Number
	 */
	itemCountPerLine: 10,
	_enabled: false,
	get enabled() {
		return this._enabled;
	},
	set enabled(value) {
		this._enabled = value;
		if (this._enabled) {
			this.show();
		} else {
			this.hide();
		}
	},
	/** 正しい値を返す cos 関数です。
	 * @param  {Number} x 数値
	 * @return {Number} 結果
	 */
	cos: function(x) {
		return Math.round(Math.cos(x) * 1000000000000000) / 1000000000000000;
	},
	/** HSL で指定された色の文字列を RGB に変換します。HSL が指定されていない場合は、ランダムに色を生成します。
	 * @param  {String} [hsl] で指定された色の文字列
	 * @return {Array} R, G, B の順に値が格納された配列
	 */
	getRGBfromHSL: function(hsl) {
		var rgb = [];
		var re = /(\d+?),(\d+?)%,(\d+?)%/;
		if (hsl) if (hsl.match(re)) {
			rgb[0] = Math.floor(parseInt(RegExp.$1) * 255 / 360 * 1.3);
			rgb[1] = Math.floor(parseInt(RegExp.$2) * 255 / 100 * 1.3);
			rgb[2] = Math.floor(parseInt(RegExp.$3) * 255 / 100 * 1.3);
			return rgb;
		}
		rgb[0] = Math.floor(Math.random() * 128);
		rgb[1] = Math.floor(Math.random() * 128);
		rgb[2] = Math.floor(Math.random() * 128);
		return rgb;
	},
	/** RGB の明度を半分にします。
	 * @param  {Array} rgbArray RGB の配列
	 * @return {Array} 結果
	 */
	rgbDarker: function(rgbArray) {
		var rgb = [];
		rgb[0] = Math.floor(rgbArray[0] / 2);
		rgb[1] = Math.floor(rgbArray[1] / 2);
		rgb[2] = Math.floor(rgbArray[2] / 2);
		return rgb;
	},
	/** ランダムな HSL 値を生成します。
	 * @return {String} HSL 値
	 */
	hslRandom: function() {
		return "hsl(" + Math.floor(Math.random() * 360) + "," + Math.floor(Math.random() * 100) + "%," + Math.floor(Math.random() * 100) + "%)"
	},
	/** HSL 値を適切な色に調節します。
	 * @param  {String} hsl HSL 値
	 * @return {String} 結果
	 */
	hslColor: function(hsl) {
		var rgb = [];		
		var re = /(\d+?)%\)/;
		if (hsl) if (hsl.match(re)) {
			return hsl.replace(RegExp.$1 + "%)", Math.min(85, 100-parseInt(RegExp.$1)) + "%)");
		}
	},
	/** HSL 値の明度を半分にします。
	 * @param  {String} hsl HSL 値
	 * @return {String} 結果
	 */
	hslDarker: function(hsl) {
		var rgb = [];		
		var re = /(\d+?)%\)/;
		if (hsl) if (hsl.match(re)) {
			return hsl.replace(RegExp.$1 + "%)", (parseInt(RegExp.$1) / 2) + "%)");
		}
	},
	/** アイテムのラベルを描画します。
	 * @param  {SVGLayer} layer レイヤー
	 * @param  {Object} circle アイテム
	 */
	drawLabel: function(layer, circle) {
		layer.strokeColor = "none";
		layer.fillColor   = "black";
		layer.fillOpacity = 1;
		var text = layer.text(circle.index, circle.x, circle.y + (this.r * 2));
		text.element.setAttributeNS(null, "text-anchor", "middle");
		text.element.setAttributeNS(null, "font-size",   "12");

		//if (circle.isAA)
		//	//console.log("AA");

		//this.addLabel(circle);
	},
	/** アイテムの円を描画します。
	 * @param  {SVGLayer} layer レイヤー
	 * @param  {Object} circle アイテム
	 */
	drawCircle: function(layer, circle) {
		layer.strokeColor = "none";
		layer.fillColor   = "black";
		layer.fillOpacity = 0.25;
		for (var i = 2; i > 0; i--) {
			layer.circle(circle.x + i, circle.y + i, circle.r + (circle.strokeWidth / 2));
		}
		layer.fillColor   = circle.fillColor;
		layer.strokeColor = circle.strokeColor;
		layer.strokeWidth = circle.strokeWidth;
		layer.fillOpacity = 1;
		layer.circle(circle.x, circle.y, circle.r);

		//if (circle.isAA)
		//	//console.log("AA");

		//this.addLabel(circle);
	},
	/** 二つのアイテム間に矢印を描画します。
	 * @param  {SVGLayer} layer    レイヤー
	 * @param  {Object} circleFrom 始点のアイテム
	 * @param  {Object} circleTo   終点のアイテム
	 */
	drawArrow: function(layer, circleFrom, circleTo) {
		layer.fillColor   = circleFrom.fillColor;
		layer.strokeColor = circleFrom.strokeColor;
		var x1 = circleFrom.x;
		var y1 = circleFrom.y;
		var x2 = circleTo.x;
		var y2 = circleTo.y;
		var r = Math.atan2(y2 - y1, x2 - x1);
		x2 -= circleTo.r * Math.cos(r);
		y2 -= circleTo.r * Math.sin(r);
		
		layer.strokeWidth = 1;
		var lx = (-6) * Math.cos(r) - (6)  * Math.sin(r);
		var ly = (-6) * Math.sin(r) + (6)  * Math.cos(r);
		var rx = (-6) * Math.cos(r) - (-6) * Math.sin(r);
		var ry = (-6) * Math.sin(r) + (-6) * Math.cos(r);
		var points = [[x1,y1],[x2 + (lx + rx) / 2, y2 + (ly + ry) / 2]];
		points.push([x2 + lx, y2 + ly]);
		points.push([x2,      y2]);
		points.push([x2 + rx, y2 + ry]);
		points.push(points[1]);
		/*
		var points = [[x1,y1],[x2,y2]];
		layer.strokeWidth = 1;	
		var lx = (-6) * Math.cos(r) - (6)  * Math.sin(r);
		var ly = (-6) * Math.sin(r) + (6)  * Math.cos(r);
		var rx = (-6) * Math.cos(r) - (-6) * Math.sin(r);
		var ry = (-6) * Math.sin(r) + (-6) * Math.cos(r);
		points.push([x2 + lx, y2 + ly]);
		points.push([x2 + rx, y2 + ry]);
		points.push([x2,      y2]);
		*/
		layer.polyline(points);
	},
	/** Moreyon を描画/表示します。*/
	show: function() {
		var resItems = ResNodes.getContainers();
 		this.width  = this.itemCountPerLine * this.r * 2; //document.body.offsetWidth;
		this.height = (Math.ceil(resItems.length / this.itemCountPerLine)+1) * this.r * 4 * 1.75; //window.innerHeight - (Nodes.header.offsetTop + Nodes.header.offsetHeight);
		this.SVGCanvas = new SVGCanvas(this.width, this.height);
		this.SVGLayers[0] = new SVGLayer("");
		this.SVGLayers[1] = new SVGLayer("");
		this.SVGLayers[2] = new SVGLayer("");
		this.SVGLayers[3] = new SVGLayer("");
		// 怪しい曲線（たぶん 0 <= y <= 1）
		var curve = [];
		for (var i = 1; i <= this.itemCountPerLine; i++) {
			//curve.push((-this.cos(((Math.PI * 0.75 * i) / this.itemCountPerLine) - (Math.PI / 2.75)) + 1) / 0.66);
			//curve.push((-Math.cos(((Math.PI * 1.5 * i) / this.itemCountPerLine) - (Math.PI / 2)) + 1) / 2);
			curve.push((Math.cos((i * 8 / this.itemCountPerLine  - Math.PI * 4) / 3)) + 1);
		}
		
		Trackback.traverse();
		var r  = this.r;
		var r2 = r * 2;
		var circles = [];
		var indexTable = [];
		var idTable = [];

		var bodyItems = ResNodes.getBodies();
		var idItems = ResNodes.getIDs();
		for (var i = 0; i < resItems.length; i++ ) {
			var resItem = resItems.items(i);
			
			var x = i % this.itemCountPerLine + 1;
			var y = Math.floor(i / this.itemCountPerLine);
			circles.push([]);
			circles[i].r = (this.r / 4) + Math.floor(resItem.textContent.length / 50);
			if (circles[i].r > this.r) circles[i].r = this.r;
			circles[i].index = Nodes.getIndexFromRes(resItem);
			
			var hsl = resItem.getAttribute("rel");
			circles[i].fillColor   = hsl == "inherit" ? this.hslRandom() : this.hslColor(hsl);
			circles[i].strokeColor = this.hslDarker(circles[i].fillColor);
			
			circles[i].strokeWidth   = r / 4;
			circles[i].x = (x * r2) - r;
			//circles[i].y = (r2 * curve[x - 1] * 3) + r + (y * r2 * 2);
			circles[i].y = (r2 * curve[x - 1] * 3) + r + (y * r2 * 3.5);
			
			var childNode = bodyItems.items(i).childNodes[0];
			if (childNode.className == "aaRes")	circles[i].isAA = true;
			if (idItems.items(i)) {
				var id = idItems.items(i).getAttribute("rel");
				if (idTable[id]) circles[i].prev = circles[idTable[id]]; //if (idTable[id]) circles[i].prev = idTable[id];
				idTable[id] = i; //idTable[id] = circles[i];
				
				// --
				indexTable["#" + circles[i].index] = i;
			}
		}

		
		for (var i = 0; i < circles.length; i++) {
			this.drawLabel(this.SVGLayers[3], circles[i]);
		}
		

		
		//console.time("draw circles");
		this.SVGLayers[1].strokeOpacity = 1;
		for (var i = 0; i < circles.length; i++) {
			this.drawCircle(this.SVGLayers[1], circles[i]);
		}
		//console.timeEnd("draw circles");
		
		this.SVGLayers[2].strokeOpacity = 0.5;
		for (var i = 0; i < circles.length; i++) {
			this
			if (circles[i].prev) {
				this.SVGLayers[2].strokeColor = circles[i].fillColor;
				this.SVGLayers[2].strokeWidth = 4;
				this.SVGLayers[2].line(circles[i].x, circles[i].y, circles[i].prev.x, circles[i].prev.y);
			}
		}
		
		//console.time("draw arrows");
		this.SVGLayers[0].strokeOpacity = 0.75;
		for (var i = 0; i < circles.length; i++) {
			if (Trackback.items[circles[i].index]) {
				for (var j = 0; j < Trackback.items[circles[i].index].length; j++) {
					var c = circles[indexTable["#" + Trackback.items[circles[i].index][j]]];
					this.SVGLayers[0].strokeOpacity = 1 - (Math.abs(c.index - circles[i].index) / 400);
					this.SVGLayers[0].fillOpacity   = this.SVGLayers[0].strokeOpacity;
					this.drawArrow(this.SVGLayers[0], c, circles[i]);
				}
			}
		}
		//console.timeEnd("draw arrows");

		this.SVGCanvas.add(this.SVGLayers[3]);
		this.SVGCanvas.add(this.SVGLayers[2]);
		this.SVGCanvas.add(this.SVGLayers[1]);
		this.SVGCanvas.add(this.SVGLayers[0]);
		this.reposition();
		this.SVGCanvas.appendTo(Nodes.scrollView);
		Nodes.scrollView.display = "block";
	},
	/** Moreyon を非表示にします。*/
	hide: function() {
		Nodes.sidePanel.style.paddingLeft = 0;
		Nodes.scrollView.removeChild(this.SVGCanvas.svg);
		Nodes.scrollView.display = "none";
	},
	/** Moreyon の位置を調節します。
	 * @ignore
	 */
	reposition: function() {
		
		Nodes.sidePanel.style.paddingLeft = this.width;
		Nodes.scrollView.style.left = 0;
		Nodes.scrollView.style.width = this.width;
	},
	/** scroll イベントを処理します。
	 * @ignore
	 */
	onScroll: function(e) {
		//console.log(e);
		this.reposition();
	}
}
/*
Moreyon.show();
Graph.addLabel("test");
*/ 

//window.addEventListener("load",   Moreyon.show.bind(Moreyon), false);
//window.addEventListener("scroll", Moreyon.onScroll.bind(Moreyon), false);