﻿/**
 * @fileOverview ポップアップを扱います。
 */

/**
 * ポップアップの要素を作成し、その情報を保持します。
 * @class
 * @param {element} source        ポップアップとして表示する要素
 * @param {Boolean} [useFade]     フェードアウトを有効にするかどうか
 * @param {Boolean} [hideOnHover] ポップアップ上でホバーしたときに非表示にするかどうか
 */
function PopupItem() {
    this.initialise.apply(this, arguments);
}
PopupItem.prototype = {
	/**  ポップアップの親要素
	 * @type element
	 */
	container:   null,
	/**  ポップアップとして表示する要素の直近の親要素
	 * @type element
　	 */
	content:     null,
	/**  ポップアップとして表示する要素
	 * @type element
	 */
	source:      null,
	/**  フェードアウトを有効にするかどうか
	 * @type Boolean
	 */
	useFade:     false,
	/**  ポップアップ上でホバーしたときに非表示にするかどうか
	 * @type Boolean
	 */
	hideOnHover: false,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
    initialise: function(source, useFade, hideOnHover) {
		var divPopupContainer          = document.createElement("div");
		divPopupContainer.className    = "popupContainer";
		var divShadowTopRight          = document.createElement("div");
		divShadowTopRight.className    = "shadowTopRight";
		var divShadowBottomLeft        = document.createElement("div");
		divShadowBottomLeft.className  = "shadowBottomLeft";
		var divShadowBottomRight       = document.createElement("div");
		divShadowBottomRight.className = "shadowBottomRight";
		var divPopupBorder             = document.createElement("div");
		divPopupBorder.className       = "popupBorder";
		var divShadowRight             = document.createElement("div");
		divShadowRight.className       = "shadowRight";
		var divShadowBottom            = document.createElement("div");
		divShadowBottom.className      = "shadowBottom";
		var divPopupBody               = document.createElement("div");
		divPopupBody.className         = "popupBody";
		
		divShadowBottom.appendChild(divPopupBody);
		divShadowRight.appendChild(divShadowBottom);
		divPopupBorder.appendChild(divShadowRight);
		divShadowBottomRight.appendChild(divPopupBorder);
		divShadowBottomLeft.appendChild(divShadowBottomRight);
		divShadowTopRight.appendChild(divShadowBottomLeft);
		divPopupContainer.appendChild(divShadowTopRight);
		
		this.source      = source;
		this.container   = divPopupContainer;
		this.content     = divPopupBody;
		this.useFade     = useFade;
		this.hideOnHover = hideOnHover;
    }
};


/**
 * 複数のポップアップをまとめて管理します。
 * @static
 */
var Popup = {
	/** PopupItem を格納した配列
	 * @type Array
	 */
	items:       new Array(),
	/** すでに初期化されたかどうかを表すフラグ
	 * @type Boolean
	 */
	initialised: false,
	/** 要素の位置とサイズを取得します。
	 * @param  {element} src 要素
	 * @return {object} left, top, width, height, right, bottom の各メンバから成るオブジェクト
	 */
	getRect: function(src) {
		var parent = src;
		var x = 0;
		var y = 0;
		while (parent) {
			x += parent.offsetLeft;
			y += parent.offsetTop;
			parent = parent.offsetParent;
		}
		if (src.parentNode.tagName == "LABEL") y += window.scrollY - 
			src.parentNode.parentNode.parentNode.scrollTop; // set y to the right position on the analyse dialog
		return {left: x, top: y, width: src.offsetWidth, height: src.offsetHeight,
		        right: x + src.offsetWidth, bottom: y + src.offsetHeight};
	},
	/** 指定された座標が要素の領域内にあるかどうかを調べます。
	 * @param  {Number}  x x 座標（スクリーン座標）
	 * @param  {Number}  y y 座標（スクリーン座標）
	 * @param  {element} element 要素
	 * @return {Boolean} 座標が要素の領域内にあれば true
	 */
	isPtInElement: function(x, y, element) {
		var rc = this.getRect(element);
		//return (x >= rc.left) && (x <= rc.right) && (y >= rc.top) && (y <= rc.bottom);
		return (x >= rc.left) && (x < rc.right) && (y >= rc.top) && (y < rc.bottom);
	},
	/** 要素がすでにポップアップとして存在しているかどうかを調べます。
	 * @param  {element} source 要素
	 * @return {Boolean} 要素がすでに存在していれば true
	 */
	sourceExists: function(source) {
		for (var i = 0; i < this.items.length; i++) {
			if (this.items[i].source == source) return true;
		}
		return false;
	},
	/** ポップアップのサイズに応じて、適切な位置に移動します。
	 * @param  {element} item 要素
	 */
	reposition: function(item) {
		var rectSource = this.getRect(item.source);
		item.container.style.left = rectSource.left;
		item.container.style.top  = rectSource.bottom;
		
		if (rectSource.bottom + item.container.offsetHeight > window.scrollY + window.innerHeight) {
			var oppositeY = (rectSource.top - item.container.offsetHeight);
			if (oppositeY >= window.scrollY + Nodes.header.offsetHeight) item.container.style.top = oppositeY;
		}
		
		if (item.container.offsetWidth > document.body.clientWidth) {
			item.container.style.left = 0;
		} else {
			if (rectSource.left + item.container.offsetWidth > window.scrollX + document.body.clientWidth)
				item.container.style.left = document.body.clientWidth - item.container.offsetWidth - window.scrollX;
		}
	},
	/** ポップアップとして要素を追加、表示します。
	 * @param  {element} content      表示する要素
	 * @param  {element} source       トリガー元の要素（位置の調整に使用される）
	 * @param  {Boolean} hideOnHover  ポップアップ上でホバーしたときに非表示にするかどうか
	 */
	add: function(content, source, hideOnHover) {
		if (this.sourceExists(source)) return;
		//var item = new PopupItem(source);
		var item = new PopupItem(source, SkinPref.getBool("enablePopupFade", true), hideOnHover);
		this.items.push(item);
		item.content.appendChild(content);
		document.body.appendChild(item.container);
		this.reposition(item);
		
		if (!this.initialised) {
			window.addEventListener("mouseout",  this.remove.bind(this), false);
			window.addEventListener("mousemove", this.remove.bind(this), false);
			this.initialised = true;
		}

		return item;
	},
	/** ポップアップを削除します。
	 * @param  {event} e イベント
	 */
	remove: function(e) {
		if (ResNodes.isContextMenuVisible()) return;
		var x = e.pageX;
		var y = e.pageY;
		for (var i = this.items.length - 1; i >= 0; i--) {
			if (this.isPtInElement(x, y, this.items[i].source))    return;
			if (!this.items[i].hideOnHover) if (this.isPtInElement(x, y, this.items[i].container)) return;
			//if (SkinPref.getBool("enablePopupFade", true)) {
			if (this.items[i].useFade) {
				this.removeChildWithFade(this.items[i].container);
			} else {
				document.body.removeChild(this.items[i].container);
			}
			this.items.pop();
		}
	},
	/** 指定された要素が載っているポップアップを探します。
	 * @param  {element} content 要素
	 * @return {PopupItem} マッチした PopupItem
	 */
	findPopup: function(content) {
		for (var i = 0; i < this.items.length; i++) {
			if (this.items[i].content.firstChild == content) return this.items[i];
		}
		return null;
	},
	/** ポップアップのフェードアウト処理を行います。removeChildWithFade() から呼ばれます。
	 * @param  {String} id 要素の ID
	 */
	fadeOutProc: function(id) {
		var node = document.getElementById(id);
		if (node) {
			var fadeStep = SkinPref.getInt("valuePopupFadeStep", 1);
			switch(fadeStep){
			case 0:
				fadeStep = 0.1;  break;
			case 1:
				fadeStep = 0.2;  break;
			case 2:
				fadeStep = 0.25; break;
			case 3:
				fadeStep = 0.33; break;
			case 4:
				fadeStep = 0.5;  break;
			default:
				fadeStep = 0.2;
			}
			node.style.opacity = parseFloat(node.style.opacity) - fadeStep;
			if (parseFloat(node.style.opacity) <= 0.0) {
				document.body.removeChild(node);
			} else {
				window.setTimeout(this.fadeOutProc.bind(this), 1, id);
			}
		}
	},
	/** ポップアップをフェードアウトしながら削除します。
	 * @param  {element} node 要素
	 */
	removeChildWithFade: function(node){
		var id = "#" + new Date().getTime() + Math.random();
		node.id = id
		node.style.opacity = 1.0;
		window.setTimeout(this.fadeOutProc.bind(this), 1, id);
	}
};

// ResPopup オブジェクト
// レス番ポップアップ（と名前欄ポップアップ）
//
/**
 * レスアンカーと名前欄のポップアップを管理します。
 * @static
 */
var ResPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** 全角文字で表記された数字を、数値型に変換します。
	 * @param  {String} str 全角数字が含まれた文字列
	 * @return {Number} 変換された数値
	 */
	toNarrow: function(str) {
		return parseInt(str.replace(/([０-９])/g, function(n){ return String.fromCharCode(n.charCodeAt(0) - 0xFEE0); }));
	},
	/** 要素を複製し、いくつかの属性を削除したものを返却します。
	 * @param  {element} src 要素
	 * @return {element} 複製された要素
	 */
	getCloneNode: function(src){
		var dest = src.cloneNode(true);
		if (dest.id) dest.id = "";
		dest.removeAttribute("name");
		dest.style.display = "block";
		var e = dest.getElementsByTagName("*");
		for(var i=0; i<e.length; i++){
			if(e[i].id) e[i].id = "";
			e[i].removeAttribute("name");
		}
		return dest;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		switch (node.className) {
			case "resPointer":
				if (node.textContent.match(/(>?>|＞)(\d{1,4})-(\d{1,4})/)) {
					this.show(node, this.toNarrow(RegExp.$2), this.toNarrow(RegExp.$3));
				} else if (node.textContent.match(/(>?>|＞)(\d{1,4})/)) {
					this.show(node, this.toNarrow(RegExp.$2), this.toNarrow(RegExp.$2));
				}
				break;
			case "resName":
				if (node.textContent.match(/^(\d{1,4})-(\d{1,4})$/)) {
					node.style.cursor = "pointer";
					this.show(node, this.toNarrow(RegExp.$1), this.toNarrow(RegExp.$2));
					node.setAttribute("onclick", "ThreadDocument.jumpTo(event)")
				} else if (node.textContent.match(/^(\d{1,4})$/)) {
					node.style.cursor = "pointer";
					this.show(node, this.toNarrow(RegExp.$1), this.toNarrow(RegExp.$1));
					node.setAttribute("onclick", "ThreadDocument.jumpTo(event)")
				}
		}
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 * @param {Number}  start  ポップアップを開始するレス番号
	 * @param {Number}  end    ポップアップを終了するレス番号
	 */
	show: function(source, start, end) {
		const POPUP_LIMIT = 20;
		if (!SkinPref.getBool("enableResPopup", true)) return;
		if (end < start) return;
		if (start < 1)  start = 1;
		if (end > 1000) end = 1000;
		if ((end - start) > POPUP_LIMIT) end = start + POPUP_LIMIT;
		Trackback.traverse();
		var content = document.createDocumentFragment();
		for(var i = start; i <= end; i++){
			resNode = Nodes.getRes(i);
			if(!resNode) {
				this.showExceeded(source, start, end);
				return;
			}
			var dlPopup = this.getCloneNode(resNode);
			dlPopup.className = "resPopup";
			Trackback.appendTo(dlPopup, start);
			content.appendChild(dlPopup);
		}
		var popupItem = Popup.add(content, source);
		// error ...
		if (popupItem) ThreadDocument.modifyAnchors(popupItem.content.parentNode);
	},
	/** 表示域外のポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 * @param {Number}  start  ポップアップを開始するレス番号
	 * @param {Number}  end    ポップアップを終了するレス番号
	 */
	showExceeded: function(source, start, end) {
	    var xmlHTTP = new XMLHttpRequest();
	    xmlHTTP.onreadystatechange = xmlEventHandler.bind(this);
	    function xmlEventHandler() {
	        if (xmlHTTP.readyState == 4) {
	            if (xmlHTTP.status == 200) {
					var content = document.createDocumentFragment();
					var divTemp  = document.createElement("div");
	                divTemp.innerHTML = xmlHTTP.responseText;
	                var dl = divTemp.getElementsByTagName("dl");
					for(var i = 0; i < dl.length; i++){
						var dlPopup = this.getCloneNode(dl[i]);
						dlPopup.className = "resPopup";
						Trackback.appendTo(dlPopup, start);
						content.appendChild(dlPopup);
					}
					var popupItem = Popup.add(content, source);
					if (popupItem) ThreadDocument.modifyAnchors(popupItem.content);
	            }
	        }
	    }
    	xmlHTTP.open("GET", SERVER_URL + EXACT_URL + start + "-" + end + "n" + "?lite=true", true);
	    xmlHTTP.send(null);
	}
};

/**
 * ID のポップアップを管理します。
 * @static
 */
var IDPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		if (SkinPref.getBool("enableIDPopupOnClick", false)) {
			window.addEventListener("click", this.onMouseOver.bind(this), false);
		} else {
			window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
		}
	},
	/** 要素を複製し、いくつかの属性を削除したものを返却します。
	 * @param  {element} src 要素
	 * @return {element} 複製された要素
	 */
	getCloneNode: function(src){
		var dest = src.cloneNode(true);
		if (dest.id) dest.id = "";
		dest.removeAttribute("name");
		dest.style.display = "block";
		var e = dest.getElementsByTagName("*");
		for(var i=0; i<e.length; i++){
			if(e[i].id) e[i].id = "";
			e[i].removeAttribute("name");
		}
		return dest;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.tagName == "SPAN") { // avoid conflict between Moreyon
			if (node.className.match(/mesID_/)) {
				var pos = node.className.indexOf(" ");
				var className = (pos == -1) ? node.className : node.className.slice(0, pos);
				IDPopup.show(node, className.slice(6), true);
			} else if (node.className.match(/^resID/)) {
				IDPopup.show(node, node.getAttribute("rel"), false);
			}
		}
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source      トリガー元の要素
	 * @param {Number}  id          ポップアップする ID
	 * @param {bool}    popupOnBody ポップアップを終了するレス番号
	 */
	show: function(source, id, popupOnBody) {
		if (!SkinPref.getBool("enableIDPopup", true)) return;
		if (id.length < 8) return;
		const IDENTIFIER = "IDPopup:";
		if (document.getElementById(IDENTIFIER + id)) return;
		ID.traverse();
		var items = ID.items[id];
		if (!items) return;
		if (items.length < (popupOnBody ? 1 : 2)) return;
		
		var appendAll = SkinPref.getBool("enableIDPopupAll", false);
		var content = document.createDocumentFragment();		
		var div = document.createElement("div");
		div.id = IDENTIFIER + id;
		div.className = "popupHeader";
		div.appendChild(appendAll ? source.cloneNode(true) : document.createTextNode("参照"));
		div.appendChild(document.createTextNode(": "));
		content.appendChild(div);
		var a = document.createElement("a");
		a.className = "resPointer";
		a.href = "javascript:void(0)";
		a.onclick = ThreadDocument.jumpTo.bind(ThreadDocument);

		for (var i = 0; i < items.length; i++) {
			if (appendAll) {
				var dlPopup = this.getCloneNode(Nodes.getRes(items[i]));
				dlPopup.className = "resPopup";
				Trackback.appendTo(dlPopup, items[i]);
				content.appendChild(dlPopup);
			}
			var anchor = a.cloneNode(false);
			anchor.appendChild(document.createTextNode(">>" + items[i]));
			div.appendChild(anchor);
		}
		
		var nodePopup = Popup.add(content, source);
		if (!appendAll) nodePopup.container.className = "popupResList";
	}
};

/**
 * 逆参照のポップアップを管理します。
 * @static
 */
var TrackbackPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** 要素を複製し、いくつかの属性を削除したものを返却します。
	 * @param  {element} src 要素
	 * @return {element} 複製された要素
	 */
	getCloneNode: function(src){
		var dest = src.cloneNode(true);
		if (dest.id) dest.id = "";
		dest.removeAttribute("name");
		dest.style.display = "block";
		var e = dest.getElementsByTagName("*");
		for(var i=0; i<e.length; i++){
			if(e[i].id) e[i].id = "";
			e[i].removeAttribute("name");
		}
		return dest;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.tagName == "A") if (node.className == "trackback") {
			if (node.textContent.match(/^(\d{1,4})$/)) {
				this.show(node, RegExp.$1);
			}
		}
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source      トリガー元の要素
	 * @param {Number}  number      ポップアップするレス番号
	 */
	show: function(source, number) {
		if (!SkinPref.getBool("enableTrackBackPopup", true)) return;
		Trackback.traverse();
		var tb = Trackback.items[number];
		if (tb.length < 1) return;
		var appendAll = SkinPref.getBool("enableTrackBackPopupAll", false) || tb.length == 1;
		var content = document.createDocumentFragment();
		if (appendAll) {
			for (var i = 0; i < tb.length; i++) {
				var node = this.getCloneNode(ResNodes.getContainerByIndex(tb[i]));
				node.className = "resPopup";
				Trackback.appendTo(node, tb[i]);
				content.appendChild(node);
			}
		} else {
			var div = document.createElement("div");
			div.className = "popupHeader";
			div.appendChild(document.createTextNode("参照: "));
			div.appendChild(Trackback.getAnchorElements(number));
			content.appendChild(div);
		}
		var nodePopup = Popup.add(content, source);
		if (!appendAll) nodePopup.container.className = "popupResList";
		if (nodePopup) ThreadDocument.modifyAnchors(nodePopup.content.parentNode);
	}
};

/**
 * 画像のポップアップを管理します。
 * @static
 */
var ImagePopup = {
	/** 画像の URI を判別する正規表現
	 * @type RegExp */
	regExp: /\.(jpg|jpeg|png|gif|mng|tiff|tif|bmp|pict)$/i,
	/** 危険なレスかどうかを判別する正規表現
	 * @type RegExp */
	grotesqueRegExp: /(ｸﾞﾛ|グロ|注意|危険|有害|ブラクラ|ﾌﾞﾗｸﾗ|精神|蓮|氏ね|死|血)/,
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** 対象の URI が画像かどうかを判別します。
	 * @param  {String}  src URI
	 * @return {Boolean} 画像であれば true
	 */
	isImage: function(src) {
		return this.regExp.test(src);
	},
	/** 対象のレス番号にグロテスク画像への注意を喚起するレスがついているか調べます。
	 * @param  {Number} index レス番号
	 * @return {Boolean} 危険なレスであれば true
	 */
	isImageGrotesque: function(index) {
		Trackback.traverse();
		var nodes = Trackback.items[index];
		if (nodes) {
			for (var i = 0; i < nodes.length; i++) {
				var node = ResNodes.getBodyByIndex(nodes[i]);
				if (node) {
					if (node.textContent.match(this.grotesqueRegExp)) {
						return true;
					}
				}
			}
		}
		return false;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.className != "outLink") return;
		//var src = node.href;
		var src = node.rel;
		//if (src.match(/\?url=/)) src = RegExp.rightContext;
		if (this.isImage(src)) {
			if (e.ctrlKey ^ SkinPref.getBool("enableDefaultPixelation", false)) {
				if (SkinPref.getInt("valuePixelationMethod", 0) == 0) {
					this.pixelateShow(node, src);
				} else {
					this.blurShow(node, src);
				}
			} else {
				if (this.isImageGrotesque(ResNodes.getIndexByBody(node.parentNode))) {
					if (SkinPref.getInt("valuePixelationMethod", 0) == 0) {
						this.pixelateShow(node, src);
					} else {
						this.blurShow(node, src);
					}
				} else {
					this.show(node, src);
				}
			}
		}
	},
	/** click イベントを処理します。
	 * @param  {event} e イベント
	 */
	onClick: function(e) {
		var node  = e.currentTarget;
		var popup = Popup.findPopup(node);
		if (popup) {
			if (node.className == "imgPopup") {
				node.className =  "imgLargePopup";
				popup.content.style.maxWidth  = "none";
				popup.content.style.maxHeight = "none";
				popup.content.style.maxWidth  = e.currentTarget.offsetWidth  - 7;
				popup.content.style.maxHeight = e.currentTarget.offsetHeight - 7;
			} else {
				if (node.tagName.match(/(canvas|svg)/i)) {
					node.parentNode.childNodes[1].style.opacity = 1;
					node.parentNode.removeChild(node);
				} else {
					e.currentTarget.className =  "imgPopup";
					popup.content.style.maxWidth  = e.currentTarget.offsetWidth  - 7;
					popup.content.style.maxHeight = e.currentTarget.offsetHeight - 7;
				}
			}
		}
	},
	/** error イベントを処理します。
	 * @param  {event} e イベント
	 */
	onError: function(e) {
		//var popup = Popup.findPopup(e.currentTarget.parentNode);
		var popup = Popup.findPopup(e.currentTarget);
		if (popup) {
			popup.source.setAttribute("title", "エラー");
		}
		e.currentTarget.parentNode.removeChild(e.currentTarget);
	},
	/** 画像のリサイズを監視します。show() から呼ばれます。画像のサイズが確定すると、ポップアップのサイズを調整し実際に表示します。
	 * @param  {PopupItem} nodePopup PopupItem オブジェクト
	 */
	resize: function(nodePopup) {
		if (!nodePopup) return;
		if (!nodePopup.content.lastChild) return;
		if ((nodePopup.content.lastChild.offsetWidth != 0) && (nodePopup.content.lastChild.offsetHeight != 0)) {
			if ((nodePopup.content.lastChild.offsetWidth != 24) && (nodePopup.content.lastChild.offsetHeight != 24)) {
			//if ((nodePopup.content.firstChild.offsetWidth != 28) && (nodePopup.content.firstChild.offsetHeight != 28)) {
				//nodePopup.container.className = "popupImage";
				//nodePopup.content.firstChild.className = "imgPopup";
				nodePopup.content.style.maxWidth  = nodePopup.content.lastChild.offsetWidth  - 7;
				nodePopup.content.style.maxHeight = nodePopup.content.lastChild.offsetHeight - 7;
				Popup.reposition(nodePopup);
				nodePopup.container.style.visibility = "visible";
				return;
			}
		}
		window.setTimeout(this.resize.bind(this), 50, nodePopup);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href      画像の URI
	 */
	show: function(source, href) {
		if (!SkinPref.getBool("enableImagePopup", true)) return;
		source.removeAttribute("title");
		var img = document.createElement("img");
		img.onerror = this.onError.bind(this);
		img.onclick = this.onClick.bind(this);
		var nodePopup = Popup.add(img, source);
		if (nodePopup) {
			nodePopup.container.className = "popupImage";
			img.className = "imgPopup";
			nodePopup.container.style.visibility = "hidden";
			nodePopup.useFade = false;
			//alert("");
			img.src = href;
			window.setTimeout(this.resize.bind(this), 50, nodePopup);
		}
	},
	/** 画像をモザイク化してポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href      画像の URI
	 */
	pixelateShow: function(source, href) {
		if (!SkinPref.getBool("enableImagePopup", true)) return;
		source.removeAttribute("title");
		var img = document.createElement("img");
		img.onerror = this.onError.bind(this);
		img.onclick = this.onClick.bind(this);
		img.style.opacity = 0.2;
		var nodePopup = Popup.add(img, source);
		var f = (function() {
			var canvas = document.createElement("canvas");
			canvas.onclick   = this.onClick.bind(this);
			canvas.style.position   = "absolute";
			canvas.style.zIndex = 1;
			canvas.width  = img.offsetWidth;
			canvas.height = img.offsetHeight;
			switch (SkinPref.getInt("valuePixelationSize", 1)) {
				case 0:
					var px = 8; break;
				case 1:
					var px = 4; break;
				case 2:
					var px = 2; break;
			}
			var ctx = canvas.getContext('2d');
			ctx.globalCompositeOperation = "copy";
			var w = canvas.width  / px;
			var h = canvas.height / px;
			ctx.drawImage(img, 0, 0, w, h);
			ctx.drawImage(canvas, 0, 0, w, h, 0, 0, canvas.width + px, canvas.height + px);
			if (SkinPref.getBool("enablePrecisePixelation", false)) {
				var m = px / 2;
				for (var y = 0; y < canvas.height; y += px) {
					for (var x = 0; x < canvas.width; x += px) {
						ctx.drawImage(canvas, x + m, y + m, 1, 1, x, y, px, px);
					}
				}
			}
			nodePopup.content.insertBefore(canvas, nodePopup.content.firstChild);
		}).bind(this);
		if (nodePopup) {
			nodePopup.container.className = "popupImage";
			img.className = "imgPopup";
			nodePopup.container.style.visibility = "hidden";
			nodePopup.useFade = false;
			img.onload = f;
			window.setTimeout(this.resize.bind(this), 50, nodePopup);
			img.src = href;
		}
	},
	/** 画像にガウスぼかしをかけてポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href      画像の URI
	 */
	blurShow: function(source, href) {
		if (!SkinPref.getBool("enableImagePopup", true)) return;
		source.removeAttribute("title");
		var img = document.createElement("img");
		img.style.opacity = 0.2;
		img.onerror = this.onError.bind(this);
		img.onclick = this.onClick.bind(this);
		var nodePopup = Popup.add(img, source);
		var f = (function() {
			switch (SkinPref.getInt("valuePixelationSize", 1)) {
				case 0:
					var px = 8; break;
				case 1:
					var px = 4; break;
				case 2:
					var px = 2; break;
			}
			var canvas = new SVGCanvas(img.offsetWidth, img.offsetHeight);
			canvas.svg.onclick = this.onClick.bind(this);
			canvas.svg.style.position  = "absolute";
			canvas.svg.style.zIndex = 1;
			var defs   = new SVGDefinitions();
			var filter = new SVGFilter("blur");
			var effect = new SVGFilterPrimitives.GaussianBlur(px);
			filter.add(effect);
			defs.add(filter);
			var layer = new SVGLayer();
			var element = layer.image(img.src, 0, 0, img.offsetWidth, img.offsetHeight);
			element.setAttribute("filter", "url(#blur)");
			canvas.add(defs);
			canvas.add(layer);
			nodePopup.content.insertBefore(canvas.svg, nodePopup.content.firstChild);
		}).bind(this);
		if (nodePopup) {
			nodePopup.container.className = "popupImage";
			img.className = "imgPopup";
			nodePopup.container.style.visibility = "hidden";
			nodePopup.useFade = false;
			img.onload = f;
			window.setTimeout(this.resize.bind(this), 50, nodePopup);
			img.src = href;
		}
	}
};

/**
 * リンク先のサムネイルプレビューのポップアップを管理します。
 * @static
 */
var UrlPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** URI がサムネイル画像へのリクエストを発行すべきか判別します。
	 * @param  {String} src URI
	 * @return {Boolean} リクエストすべきであれば true
	 */
	isValid: function(src) {
		if (src.match(/(http|https):\/\/.+\/.*\.([a-zA-Z0-9]+)$/)) {
			var ext = RegExp.$2;
			return ext.match(/^(htm|html|shtm|shtml|stm|xml|xhtml|php|php3|php4|cgi|jsp|cfm|asp|aspx|pl|plx|py|rb|)$/);
		} else {
			return true;
		}
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.className != "outLink") return;
		//var src = node.href;
		var src = node.rel;
		//if (src.match(/\?url=/)) src = RegExp.rightContext;
		if (src.match(/2ch.net/)) return;
		if (src.match(/read\.cgi/)) return;
		if (ImagePopup.isImage(src)) return;
		if (VideoPopup.getVideoSource(src)) return;
		if (this.isValid(src)) this.show(node, src);
	},
	/** error イベントを処理します。
	 * @param  {event} e イベント
	 */
	onError: function(e) {
		var popup = Popup.findPopup(e.currentTarget.parentNode);
		if (popup) {
			popup.source.setAttribute("title", "エラー");
		}
		e.currentTarget.parentNode.removeChild(e.currentTarget);
	},
	/** 画像のリサイズを監視します。show() から呼ばれます。画像のサイズが確定すると、ポップアップのサイズを調整し実際に表示します。
	 * @param  {PopupItem} nodePopup PopupItem オブジェクト
	 */
	resize: function(nodePopup) {
		if (!nodePopup) return;
		if (!nodePopup.content.firstChild.firstChild) return;
		if ((nodePopup.content.firstChild.firstChild.offsetWidth != 0) && (nodePopup.content.firstChild.firstChild.offsetHeight != 0)) {
			if ((nodePopup.content.firstChild.firstChild.offsetWidth != 28) && (nodePopup.content.firstChild.firstChild.offsetHeight != 28)) {
				nodePopup.container.className = "popupImage";
				nodePopup.content.firstChild.firstChild.className = "urlPopup";
				nodePopup.content.style.maxWidth  = nodePopup.content.firstChild.firstChild.offsetWidth  - 7;
				nodePopup.content.style.maxHeight = nodePopup.content.firstChild.firstChild.offsetHeight - 7;
				Popup.reposition(nodePopup);
				nodePopup.container.style.visibility = "visible";
				return;
			}
		}
		window.setTimeout(this.resize.bind(this), 10, nodePopup);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href     リンク先の URI
	 */
	show: function(source, href) {
		if (!SkinPref.getBool("enableUrlPopup", true)) return;
		source.removeAttribute("title");
		var a    = document.createElement("a");
		a.href   = href;
		if (SkinPref.getBool("enableNewWindow", true)) a.target = "_blank";
		var img = document.createElement("img");
		img.src = "http://shots.snap.com/preview/?url=" + encodeURIComponent(href) + "&key=9101c9d3294ad9f8c6627143df504a3a&src=www.snap.com&cp=&sb=1&v=2.19.1&size=" +
			(SkinPref.getInt("valueUrlPopupSize", 0) == 0 ? "large" : "small");
		img.setAttribute("onerror", "UrlPopup.onError(event)");
		a.appendChild(img);
		var nodePopup = Popup.add(a, source, true);
		if (nodePopup) {
			nodePopup.container.style.visibility = "hidden";
			window.setTimeout(this.resize.bind(this), 10, nodePopup);
		}
	}
};


/**
 * 動画サイトのポップアップを管理します。
 * @static
 */
var VideoPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** URI から動画のプレビューのための正しいソース URI を取得します。
	 * @param  {String} src URI
	 * @return {String} ソース URI
	 */
	getVideoSource: function(src) {
		//if (src.match(/youtube\.com\/.*\?v=([^\&]+)/)) {
		if (src.match(/youtube\.com\/watch\?v=([^&]+)/)) {
			var option = SkinPref.getBool("videoPopupAutoStart", true) ? "&autoplay=1" : "";
      		return "http://www.youtube.com/v/" + RegExp.$1 + option;
	    } else if (src.match(/stage6.divx.com\/.*\?content_id=(\d+)/)) {
	    	return "http://images.stage6.com/videos/" + RegExp.$1 + ".jpg";
	    } else if (src.match(/stage6.divx.com\/.*\/show_video\/(\d+)/)) {
	    	return "http://images.stage6.com/videos/" + RegExp.$1 + ".jpg";
	    } else if (src.match(/stage6.divx.com\/members\/(\d+)\/videos\/(\d+)/)) {
	    	return "http://video.stage6.com/" + RegExp.$1 + "/" + RegExp.$2 + ".divx";
	    } else if (src.match(/video.google.*\?docid=([\d\-]+)/)) {
	    	return "http://video.google.com/googleplayer.swf?docId=" + RegExp.$1;
	    } else if (src.match(/metacafe.com\/watch\/(\d+)/)){
	    	return "http://www.metacafe.com/fplayer/" + RegExp.$1 + ".swf";
	    } else if (src.match(/guba.com\/watch\/(\d+)/)){
	    	return "http://www.guba.com/f/root.swf?video_url=http://free.guba.com/uploaditem/" + RegExp.$1 + "/flash.flv&isEmbeddedPlayer=true";
	    } else if (src.match(/nicovideo.jp\/watch\/([^\&]+)/)){
	    	//return "http://nicopon.jp/video/FlowPlayer.swf?config={videoFile:%20'http://nicopon.jp/video/src/" + RegExp.$1 + "',%20initialScale:'fit'}";
	    	return "http://www.nicovideo.jp/thumb/" + RegExp.$1
	    } else if (src.match(/nicovideo.jp\/.*\?v=([^\&]+)/)){
	    	//return "http://nicopon.jp/video/FlowPlayer.swf?config={videoFile:%20'http://nicopon.jp/video/src/" + RegExp.$1 + "',%20initialScale:'fit'}";
	    	return "http://www.nicovideo.jp/thumb/" + RegExp.$1
	    } else {
	    	return false;
	    }
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.className != "outLink") return;
		//var src = node.href;
		var src = node.rel;
		//if (src.match(/\?url=/)) src = RegExp.rightContext;
		if (ImagePopup.isImage(src)) return;
		if (src.match(/2ch.net/)) return;
		var href = this.getVideoSource(src);
		if (href) this.show(node, href);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href     ソース URI
	 */
	show: function(source, href) {
		if (!SkinPref.getBool("enableVideoPopup", true)) return;
		if (href.match(/\.jpg$/)) {
			//ImagePopup.show(source, href, source.href);
			ImagePopup.show(source, href);
			return;
		} else if (href.match(/nicovideo\.jp/)) {
			var iframe = document.createElement("iframe");
			iframe.width = 340;
			iframe.height = 200;
			iframe.src = href;
			var nodePopup = Popup.add(iframe, source);
			if (nodePopup) {
				nodePopup.container.className = "popupIframe";
				nodePopup.content.style.maxWidth  = iframe.width  - 7;
				nodePopup.content.style.maxHeight = iframe.height - 7;
				Popup.reposition(nodePopup);
			}
		} else {
			var obj = document.createElement("object");
			obj.setAttribute("type", href.match(/\.divx$/) ? "video/divx" : "application/x-shockwave-flash");
			obj.setAttribute("data", href);
			obj.setAttribute("width",  "340");
			obj.setAttribute("height", "288");
			var param = document.createElement("param");
			param.setAttribute("name",  "wmode");
			param.setAttribute("value", "transparent");
			obj.appendChild(param);
			var nodePopup = Popup.add(obj, source);
			if (nodePopup) {
				nodePopup.container.className = "popupVideo";
				Popup.reposition(nodePopup);
			}
		}
	}
};


/**
 * フッタの「おすすめ２ちゃんねる」のポップアップを管理します。
 * @static
 */
var Osusume2chPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.id != "osusume2ch") return;
		this.show(node);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 */
	show: function(source) {
		var iframe = document.createElement("iframe");
		iframe.width = 500;
		iframe.height = 200;
		iframe.src = "http://" + ThreadDocument.host + "/" + ThreadDocument.boardName + "/i/" + ThreadDocument.threadID + ".html";
		var nodePopup = Popup.add(iframe, source);
		if (nodePopup) {
			nodePopup.container.className = "popupIframe";
			nodePopup.content.style.maxWidth  = iframe.width  - 7;
			nodePopup.content.style.maxHeight = iframe.height - 7;
			Popup.reposition(nodePopup);
		}
	}
};

/**
 * フッタの「関連キーワード」のポップアップを管理します。
 * @static
 */
var RelatedKeywordsPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.id != "relatedKeywords") return;
		this.show(node);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 */
	show: function(source) {
		var iframe = document.createElement("iframe");
		iframe.width = 400;
		iframe.style.height = "1.5em";
		iframe.src = "http://p2.2ch.io/getf.cgi?" + EXACT_URL;
		var nodePopup = Popup.add(iframe, source);
		if (nodePopup) {
			nodePopup.container.className = "popupIframe";
			nodePopup.content.style.maxWidth  = iframe.offsetWidth  - 7;
			nodePopup.content.style.maxHeight = iframe.offsetHeight - 7;
			Popup.reposition(nodePopup);
		}
	}
};
