﻿// svgcanvas.js
// SVG 動的生成サポートライブラリ
//
// 簡単な使い方:
//
// (1) SVGCanvas オブジェクトを生成
//   var canvas = new SVGCanvas(width, height);
// (2) SVGLayer オブジェクトを生成
//   var layer = new SVGLayer(id);
// (3) レイヤに描画
//   layer.fillColor = "black";
//   layer.text("Hello, world!", 0, 0); // <canvas> ﾌﾟｷﾞｬｰ
// (4) キャンバスにレイヤを追加
//   canvas.addLayer(layer);
// (5) 要素にキャンバスを追加
//   canvas.appendTo(document.body);
//
// * <canvas>なんてウンコは使ってられないぜ
// * SVGAnchor を使えばリンクも設定できるぜ
// * 細かい属性は思いっきり端折ってやったぜ
// 


/**
 * @fileOverview SVG の動的生成を補助するライブラリです。
 */
 
 /**
 * svg 要素を管理します。
 * @class
 * @param {Number} width  幅
 * @param {Number} height 高さ
 */
function SVGCanvas() {
    this.initialise.apply(this, arguments);
}
SVGCanvas.prototype = {
	SVG_NS: 'http://www.w3.org/2000/svg',
	XLink_NS: 'http://www.w3.org/1999/xlink',
	/** svg 要素
	 * @type element
	 */
	svg: null,
	/** 幅
	 * @type Number
	 */
	width: null,
	/** 高さ
	 * @type Number
	 */
	height: null,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(width, height) {
		this.svg = document.createElementNS(this.SVG_NS, "svg");
		//this.svg.setAttributeNS(null, "xmlns:xlink",  this.XLink_NS);
		this.svg.setAttributeNS(null, "width",  width);
		this.svg.setAttributeNS(null, "height", height);
		this.width  = width;
		this.height = height;
	},
	/** 指定された要素に子要素として追加します。
	 * @param {element} x 親要素
	 */
	appendTo: function(x) {
		x.appendChild(this.svg);
	},
	/** 指定された要素から削除します。
	 * @param {element} x 親要素
	 */
	removeFrom: function(x) {
		x.removeChild(this.svg);
	},
	/** 子要素を追加します。
	 * @param {Object} x 親要素
	 */
	add: function(x) {
		this.svg.appendChild(x.element);
	},
	/** 子要素を削除します。
	 * @param {Object} x 親要素
	 */
	remove: function(x) {
		this.svg.removeChild(x.element);
	}
};

 /**
 * SVG の各種子要素を管理します。
 * @class
 * @param {Number} width  幅
 * @param {Number} height 高さ
 * @property {String} id ID
 * @property {String} strokeColor   ストロークの色
 * @property {Number} strokeWidth   ストロークの幅
 * @property {Number} strokeOpacity ストロークの透明度
 * @property {String} fillColor   塗りつぶしの色
 * @property {Number} fillOpacity 塗りつぶしの透明度
 */
function SVGElement() {
    this.initialise.apply(this, arguments);
}
SVGElement.prototype = {
	SVG_NS: 'http://www.w3.org/2000/svg',
	XLink_NS: 'http://www.w3.org/1999/xlink',
	/** 要素
	 * @type element
	 */
	element: null,
	/** 要素名
	 * @type String
	 */
	name: null,
	_id: null,
	_strokeWidth: null,
	_strokeColor: null,
	_strokeOpacity: null,
	_fillColor: null,
	_fillOpacity: null,
	get id()  { return this._id; },
	set id(x) { this.element.setAttributeNS(null, "id", x); this._id = x; },
	get strokeColor()    { return this._strokeColor },
	set strokeColor(x)   { this.element.setAttributeNS(null, "stroke", x); this._strokeColor = x; },
	get strokeWidth()    { return this._strokeWidth },
	set strokeWidth(x)   { this.element.setAttributeNS(null, "stroke-width", x); this._strokeWidth = x; },
	get strokeOpacity()  { return this._strokeOpacity },
	set strokeOpacity(x) { this.element.setAttributeNS(null, "stroke-opacity", x); this._strokeOpacity = x; },
	get fillColor()      { return this._fillColor },
	set fillColor(x)     { this.element.setAttributeNS(null, "fill", x); this._fillColor = x; },
	get fillOpacity()    { return this._fillOpacity },
	set fillOpacity(x)   { this.element.setAttributeNS(null, "fill-opacity", x); this._fillOpacity = x; },
	/*
	_onClick:     null,
	_onMouseDown: null,
	_onMouseUp:   null,
	_onMouseMove: null,
	_onMouseOut:  null,
	_onMouseOver: null,
	_onActivate:  null,
	get onClick()      { return this._onClick },
	set onClick(x)     { this.element.setAttributeNS(null, "onclick", x); this._onClick = x; },
	get onMouseDown()  { return this._onMouseDown },
	set onMouseDown(x) { this.element.setAttributeNS(null, "onmousedown", x); this._onMouseDown = x; },
	get onMouseUp()    { return this._onMouseUp },
	set onMouseUp(x)   { this.element.setAttributeNS(null, "onmouseup", x); this._onMouseUp = x; },
	get onMouseMove()  { return this._onMouseMove },
	set onMouseMove(x) { this.element.setAttributeNS(null, "onmousemove", x); this._onMouseMove = x; },
	get onMouseOut()   { return this._onMouseOut },
	set onMouseOut(x)  { this.element.setAttributeNS(null, "onmouseout", x); this._onMouseOut = x; },
	get onMouseOver()  { return this._onMouseOver },
	set onMouseOver(x) { this.element.setAttributeNS(null, "onmouseover", x); this._onMouseOver = x; },
	get onActivate()   { return this._onActivate },
	set onActivate(x)  { this.element.setAttributeNS(null, "onactivate", x); this._onActivate = x; },
	*/
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(name, layer, stroke, fill) {
		this.name = name;
		this.element = document.createElementNS(this.SVG_NS, name);
		if (stroke) {
			this.strokeColor   = layer.strokeColor   ? layer.strokeColor   : "none"
			this.strokeWidth   = layer.strokeWidth   ? layer.strokeWidth   : "1"
			this.strokeOpacity = layer.strokeOpacity ? layer.strokeOpacity : "1"
		}
		if (fill) {
			this.fillColor     = layer.fillColor     ? layer.fillColor     : "none"
			this.fillOpacity   = layer.fillOpacity   ? layer.fillOpacity   : "1"
		}
	},
	/** 属性の値を取得します。
	 * @param  {String} name 属性名
	 * @return {undefined} 値
	 */
	getAttribute: function(name) {
		this.element.getAttributeNS(null, name);
	},
	/** 属性に値を設定します。
	 * @param {String}    name  属性名
	 * @param {undefined} value 値
	 */
	setAttribute: function(name, value) {
		this.element.setAttributeNS(null, name, value);
	}
};

/**
 * SVG のa 要素を管理します。
 * @class
 * @param {String} href URI
 * @param {String} id   ID
 */
function SVGAnchor() {
    this.initialise.apply(this, arguments);
}
SVGAnchor.prototype = {
	SVG_NS: 'http://www.w3.org/2000/svg',
	XLink_NS: 'http://www.w3.org/1999/xlink',
	/** 要素
	 * @type element
	 */
	element: null,
	/** ID
	 * @type String
	 */
	id: null,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(href, id) {
		this.element = document.createElementNS(this.SVG_NS, "a");
		this.element.setAttributeNS(this.XLink_NS, "xlink:href",  href);
		this.element.setAttributeNS(null, "id", id);
		this.id = id;
	},
	/** 要素を追加します。
	 * @param {Object} x 要素
	 */
	add: function(x) {
		this.element.appendChild(x.element);
	},
	/** 要素を削除します。
	 * @param {Object} x 要素
	 */
	remove: function(x) {
		this.element.removeChild(x.element);
	}
};

/**
 * SVG の g 要素を管理します。
 * @class
 * @param {String} id ID
 */
function SVGLayer() {
    this.initialise.apply(this, arguments);
}
SVGLayer.prototype = {
	SVG_NS: 'http://www.w3.org/2000/svg',
	XLink_NS: 'http://www.w3.org/1999/xlink',
	/** 要素
	 * @type element
	 */
	element: null,
	/** ID
	 * @type String
	 */
	id: null,
    /* ストロークの幅
     * @type Number
     */
	strokeWidth: null,
    /* ストロークの色
     * @type String
     */
	strokeColor: null,
    /* ストロークの透明度
     * @type Number
     */
	strokeOpacity: null,
    /* 塗りつぶしの色
     * @type String
     */
	fillColor: null,
    /* 塗りつぶしの透明度
     * @type Number
     */
	fillOpacity: null,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(id) {
		this.element = document.createElementNS(this.SVG_NS, "g");
		this.element.setAttributeNS(null, "id",  id);
		this.id = id;
		return this.element;
	},
	/** 配列からカンマ区切りの座標リストへ変換します。
	 * @param  {Array} x 配列
     * @return {String} 文字列
	 */
	arrayToPoints: function(x) {
		var p = [];
		for (var i = 0; i < x.length; i++)
			p[i] = x[i].join(" ");
		return p.join(",");
	},
	/** 要素を追加します。
	 * @param {Object} x 要素
	 */
	add: function(x) {
		this.element.appendChild(x.element);
	},
	/** 要素を削除します。
	 * @param {Object} x 要素
	 */
	remove: function(x) {
		this.element.removeChild(x.element);
	},
	/*
	createSVGElement: function(name, stroke, fill) {
		var element = document.createElementNS(this.SVG_NS, name);
		if (stroke) {
			element.setAttributeNS(null, "stroke", this.strokeColor ? this.strokeColor : "none");
			element.setAttributeNS(null, "stroke-width", this.strokeWidth ? this.strokeWidth : "1");
			element.setAttributeNS(null, "stroke-opacity", this.strokeOpacity ? this.strokeOpacity : "1");
		}
		if (fill) {
			element.setAttributeNS(null, "fill", this.fillColor   ? this.fillColor   : "none");
			element.setAttributeNS(null, "fill-opacity", this.fillOpacity ? this.fillOpacity : "1");
		}
		return element;
	},
	*/
	/** 四角形を作成します。
	 * @param  {Number} x X 座標
	 * @param  {Number} y Y 座標
	 * @param  {Number} width  幅
	 * @param  {Number} height 高さ
     * @return {SVGElement} SVGElement オブジェクト
	 */
	rect: function(x, y, width, height) {
		//var rect = this.createSVGElement("rect", true, true);
		var rect = new SVGElement("rect", this, true, true);
		rect.element.setAttributeNS(null, "x",      x);
		rect.element.setAttributeNS(null, "y",      y);
		rect.element.setAttributeNS(null, "width",  width);
		rect.element.setAttributeNS(null, "height", height);
		this.element.appendChild(rect.element);
		return rect;
	},
	/** 円を作成します。
	 * @param  {Number} cx 中心の X 座標
	 * @param  {Number} cy 中心の Y 座標
	 * @param  {Number} r  半径
     * @return {SVGElement} SVGElement オブジェクト
	 */
	circle: function(cx, cy, r) {
		var circle = new SVGElement("circle", this, true, true);
		circle.element.setAttributeNS(null, "cx", cx);
		circle.element.setAttributeNS(null, "cy", cy);
		circle.element.setAttributeNS(null, "r",  r);
		this.element.appendChild(circle.element);
		return circle;
	},
	/** 楕円を作成します。
	 * @param  {Number} cx 中心の X 座標
	 * @param  {Number} cy 中心の Y 座標
	 * @param  {Number} rx X 軸方向の半径
	 * @param  {Number} ry Y 軸方向の半径
     * @return {SVGElement} SVGElement オブジェクト
	 */
	ellipse: function(cx, cy, rx, ry) {
		var ellipse = new SVGElement("ellipse", this, true, true);
		ellipse.element.setAttributeNS(null, "cx", cx);
		ellipse.element.setAttributeNS(null, "cy", cy);
		ellipse.element.setAttributeNS(null, "rx", rx);
		ellipse.element.setAttributeNS(null, "ry", ry);
		this.element.appendChild(ellipse.element);
		return ellipse;
	},
	/** 直線を作成します。
	 * @param  {Number} x1 始点の X 座標
	 * @param  {Number} y1 始点の Y 座標
	 * @param  {Number} x2 始点の X 座標
	 * @param  {Number} y2 終点の Y 座標
     * @return {SVGElement} SVGElement オブジェクト
	 */
	line: function(x1, y1, x2, y2) {
		var line = new SVGElement("line", this, true, false);
		line.element.setAttributeNS(null, "x1", x1);
		line.element.setAttributeNS(null, "y1", y1);
		line.element.setAttributeNS(null, "x2", x2);
		line.element.setAttributeNS(null, "y2", y2);
		this.element.appendChild(line.element);
		return line;
	},
	/** 折れ線を作成します。
	 * @param  {Array/String} points 座標のリスト
     * @return {SVGElement} SVGElement オブジェクト
	 */
	polyline: function(points) {
		//var polyline = new SVGElement("polyline", this, true, false);
		var polyline = new SVGElement("polyline", this, true, true);
		polyline.element.setAttributeNS(null, "points", (points instanceof Array) ? this.arrayToPoints(points) : points);
		this.element.appendChild(polyline.element);
		return polyline;
	},
	/** 多角形を作成します。
	 * @param  {Array/String} points 座標のリスト
     * @return {SVGElement} SVGElement オブジェクト
	 */
	polygon: function(points) {
		var polygon = new SVGElement("polygon", this, true, true);
		polygon.element.setAttributeNS(null, "points", (points instanceof Array) ? this.arrayToPoints(points) : points);
		this.element.appendChild(polygon.element);
		return polygon;
	},
	/** 文字列を作成します。
	 * @param  {String} str 文字列
	 * @param  {Number} x X 座標
	 * @param  {Number} y Y 座標
     * @return {SVGElement} SVGElement オブジェクト
	 */
	text: function(str, x, y) {
		var text = new SVGElement("text", this, true, true);
		text.element.setAttributeNS(null, "x", x);
		text.element.setAttributeNS(null, "y", y);
		text.element.appendChild(document.createTextNode(str));
		this.element.appendChild(text.element);
		return text;
	},
	/** 画像を作成します。
	 * @param  {String} src URI
	 * @param  {Number} x X 座標
	 * @param  {Number} y Y 座標
	 * @param  {Number} width  幅
	 * @param  {Number} height 高さ
     * @return {SVGElement} SVGElement オブジェクト
	 */
	image: function(src, x, y, width, height) {
		var image = new SVGElement("image", this, false, false);
		image.element.setAttributeNS(this.XLink_NS, "xlink:href", src);
		if (x) image.element.setAttributeNS(null, "x", x);
		if (y) image.element.setAttributeNS(null, "y", y);
		if (width)  image.element.setAttributeNS(null, "width",  width);
		if (height) image.element.setAttributeNS(null, "height", height);
		this.element.appendChild(image.element);
		return image;
	},
};

/**
 * SVG の defs 要素を管理します。
 * @class
 */
function SVGDefinitions() {
    this.initialise.apply(this, arguments);
}
SVGDefinitions.prototype = {
	SVG_NS: 'http://www.w3.org/2000/svg',
	XLink_NS: 'http://www.w3.org/1999/xlink',
	/** 要素
	 * @type element
	 */
	element: null,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function() {
		this.element = document.createElementNS(this.SVG_NS, "defs");
		return this.element;
	},
	/** 要素を追加します。
	 * @param {Object} x 要素
	 */
	add: function(x) {
		this.element.appendChild(x.element);
	},
	/** 要素を削除します。
	 * @param {Object} x 要素
	 */
	remove: function(x) {
		this.element.removeChild(x.element);
	}
};

/**
 * SVG の filter 要素を管理します。
 * @param {String} id     ID
 * @param {Number} x      X 座標
 * @param {Number} y      Y 座標
 * @param {Number} width  幅
 * @param {Number} height 高さ
 * @class
 */
function SVGFilter() {
    this.initialise.apply(this, arguments);
}
SVGFilter.prototype = {
	SVG_NS: 'http://www.w3.org/2000/svg',
	XLink_NS: 'http://www.w3.org/1999/xlink',
	/** 要素
	 * @type element
	 */
	element: null,
	/** ID
	 * @type String
	 */
	id: null,
	/** X 座標
	 * @type Number
	 */
	x: 0,
	/** Y 座標
	 * @type Number
	 */
	y: 0,
	/** 幅
	 * @type Number
	 */
	width:  0,
	/** 高さ
	 * @type Number
	 */
	height: 0,
	effects: [],
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(id, x, y, width, height) {
		this.element = document.createElementNS(this.SVG_NS, "filter");
		this.element.setAttributeNS(null, "id",  id);
		this.id = id;
		if (x) this.element.setAttributeNS(null, "x",  x);
		if (y) this.element.setAttributeNS(null, "y",  y);
		if (width)  this.element.setAttributeNS(null, "width",   width);
		if (height) this.element.setAttributeNS(null, "height",  height);
		return this.element;
	},
	/** 要素を追加します。
	 * @param {Object} x 要素
	 */
	add: function(x) {
		this.element.appendChild(x.element);
	},
	/** 要素を削除します。
	 * @param {Object} x 要素
	 */
	remove: function(x) {
		this.element.removeChild(x.element);
	}
};

/**
 * SVG の 原始フィルタを提供します。
 * @static
 */
var SVGFilterPrimitives = {
	SVG_NS: 'http://www.w3.org/2000/svg',
	XLink_NS: 'http://www.w3.org/1999/xlink',
	/** 原始フィルタを作成します。
	 * @param {String} name     フィルタ名
	 * @param {String} [x]      X 座標
	 * @param {String} [y]      Y 座標
	 * @param {String} [input]  入力
	 * @param {String} [result] 出力
	 */
	create: function(name, x, y, input, result) {
		var element = document.createElementNS(this.SVG_NS, name);
		if (x) element.setAttributeNS(null, "x", x);
		if (y) element.setAttributeNS(null, "y", y);
		if (input)  element.setAttributeNS(null, "in", input);
		if (result) element.setAttributeNS(null, "result", result);
		return element;
	}
};
/**
 * SVG の feGaussianBlur 要素を管理します。
 * @class
 * @param {Number} stdDeviation 標準偏差
 * @param {String} [x]      X 座標
 * @param {String} [y]      Y 座標
 * @param {String} [input]  入力
 * @param {String} [result] 出力
 */
SVGFilterPrimitives.GaussianBlur = function() {
	this.initialise.apply(this, arguments);
}
SVGFilterPrimitives.GaussianBlur.prototype = {
	/** 要素
	 * @type element
	 */
	element: null,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(stdDeviation, x, y, input, result) {
		this.element = SVGFilterPrimitives.create("feGaussianBlur", x, y, input, result);
		this.element.setAttributeNS(null, "stdDeviation", stdDeviation);
	}
}